// Load the page metrics on a web page and adds them to the current page loading interaction.

function getNavigationStartTime() {
    return window.webclient && window.webclient.getRealClickTime ? window.webclient.getRealClickTime() : 0;
}

let interaction = {
    target: 'webview',
    time: getNavigationStartTime(),
    timeline: [],
    tags: []
};

const maxRetryCount = 14;

function addEvent(name, when) {
    if (interaction.time !== 0) {
        interaction.timeline.push({name: name, offset: when - interaction.time});
    }
}

function addEventNow(name) {
    addEvent(name, Date.now());
}

function measureFirstPaint(retryCount) {
    let performance = window.performance;
    let performanceEntries = performance.getEntriesByType('paint');

    if (performanceEntries.length === 0 && retryCount < maxRetryCount) {
        retryCount++;
        setTimeout(measureFirstPaint, 300, retryCount);
        return;
    }

    addEvent('t0', window.ue_t0);
    performanceEntries.forEach( (performanceEntry, i, entries) => {
        addEvent(performanceEntry.name, Date.now() - performance.now() + performanceEntry.startTime);
    });
    interaction.tags.push({name: 'timeOrigin', value: performance.timeOrigin});
    interaction.tags.push({name: 'performanceNow', value: performance.now()});
    interaction.tags.push({name: 'DateNow', value: Date.now()});
    interaction.tags.push({name: 'paintMetricsRetryCount', value: retryCount});
    sendInteraction();
}

function sendInteraction() {
    if (window.webclient && window.webclient.send) {
        window.webclient.send(JSON.stringify({
            topic: 'interaction.addToCurrent',
            data: interaction
        }));
    }
}

function measure(retryCount) {
    var dateNow = Date.now();
    if ((document.readyState === 'loading' || !window.ue_pty
            || !window.performance.getEntriesByType('navigation')
            || !window.performance.getEntriesByType('navigation')[0]
            || typeof window.performance.getEntriesByType('navigation')[0].transferSize === "undefined")
            && retryCount < maxRetryCount) {
        retryCount++;
        setTimeout(measure, 300, retryCount);
        return;
    }
    let transferSize = -1;
    let dataSource = 'unknown';
    if (window.performance.getEntriesByType('navigation')
                    && window.performance.getEntriesByType('navigation')[0]
                    && typeof window.performance.getEntriesByType('navigation')[0].transferSize !== "undefined") {
        transferSize = window.performance.getEntriesByType('navigation')[0].transferSize;
        dataSource = (transferSize === 0 ? 'disk' : 'origin');
    }

    // pty: pageType spty: subPageType pti: pageId
    interaction.tags.push({name: 'pty', value: window.ue_pty});
    interaction.tags.push({name: 'spty', value: window.ue_spty});
    interaction.tags.push({name: 'pti', value: window.ue_pti});
    interaction.tags.push({name: 'url', value: window.location.href});
    interaction.tags.push({name: 'onMeasureRetryCount', value: retryCount});

    if(args.hasOwnProperty('dataSource')) {
        interaction.tags.push({name: 'dataSource', value: args.dataSource});
        switch (args.dataSource) {
            case 'memory':
                interaction.tags.push({name: 'isCached', value: true});
                interaction.tags.push({name: 'transferSize', value: 0});
                interaction.tags.push({name: 'originalDataSource', value: dataSource});
                interaction.tags.push({name: 'originalTransferSize', value: transferSize});
                addEvent('first-paint', args.firstPaint);
                sendInteraction();
                break;
            case 'jumpStart':
                interaction.tags.push({name: 'isCached', value: false});
                measureFirstPaint(0);
                break;
        }
    } else {
        interaction.tags.push({name: 'dataSource', value: dataSource});
        interaction.tags.push({name: 'transferSize', value: transferSize});
        interaction.tags.push({name: 'isCached', value: transferSize === 0});
        measureFirstPaint(0);
    }
}

measure(0);


